@@ -2,6 +2,7 @@
 """
 from __future__ import annotations
 
+import ctypes
 import sys
 from functools import lru_cache as cache
 from functools import partial
@@ -37,9 +38,9 @@ if sys.version_info.minor < 9: # pragma: no cover
 class SimpleQueue(_SimpleQueue[T]):
     def __init__(self, *args):
         super().__init__(*args, ctx=multiprocessing.get_context('fork'))
-    def put(self, element: T):
+    def put(self, obj: T):
         try:
-            super().put(element)
+            super().put(obj)
         except PicklingError:
             raise # pragma: no cover
         # https://bugs.python.org/issue29187
@@ -50,6 +51,13 @@ class SimpleQueue(_SimpleQueue[T]):
             raise PicklingError(message)
     def close(self): # Python 3.8 static typing trick
         super().close() # type: ignore
+    def wlock_release(self):
+        if (lock := getattr(self, '_wlock', None)) is None:
+            return # pragma: no cover
+        try:
+            lock.release()
+        except ValueError:
+            pass
 
 
 def drop_params(fn: Callable[[], T]) -> Callable[..., T]:
@@ -70,4 +78,8 @@ def gradio_request_var():
     return LocalContext.request
 
 
+def malloc_trim():
+    ctypes.CDLL("libc.so.6").malloc_trim(0)
+
+
 debug = partial(print, 'SPACES_ZERO_GPU_DEBUG')
