@@ -15,8 +15,10 @@ from typing_extensions import assert_never
 from .. import utils
 from ..config import Config
 from .api import APIClient
+from .api import AuthLevel
 from .api import QuotaInfos
 from .api import ScheduleResponse
+from .gradio import HTMLError
 from .gradio import get_event
 from .gradio import supports_auth
 
@@ -26,11 +28,13 @@ DEFAULT_SCHEDULE_DURATION = 60
 
 QUOTA_MESSAGE = "You have exceeded your GPU quota"
 UNUSED_MESSAGE = "GPU device not used"
-NO_GPU_MESSAGE_REGULAR = "No GPU is currently available"
-NO_GPU_MESSAGE_INQUEUE = "No GPU is currently available for you after 60s"
+NO_GPU_MESSAGE_REGULAR = "No GPU was available"
+NO_GPU_MESSAGE_INQUEUE = "No GPU was available after 60s"
 
-SUBSCRIBE_TO_PRO_URL = "https://huggingface.co/subscribe/pro"
+SIGNUP_ON_HF_TXT = "Create a free account"
 SIGNUP_ON_HF_URL = "https://huggingface.co/join"
+SUBSCRIBE_TO_PRO_TXT = "Subscribe to Pro"
+SUBSCRIBE_TO_PRO_URL = "https://huggingface.co/settings/billing/subscription"
 
 
 def api_client():
@@ -50,6 +54,43 @@ def startup_report():
         raise RuntimeError("Error while initializing ZeroGPU: Unknown")
 
 
+def html_string(html_contents: str, text_contents: str): # pragma: no cover
+    class HTMLString(str):
+        def __str__(self):
+            return text_contents
+    return HTMLString(html_contents)
+
+
+def _toast_action(
+    auth: AuthLevel | None,
+    supports_html: bool,
+    pro_message: str,
+    unlogged_desc: str,
+    logged_desc: str,
+    ending: str,
+) -> tuple[str, str]: # pragma: no cover
+    if not supports_auth() or auth == 'pro':
+        return pro_message, pro_message
+    html = ""
+    link = SIGNUP_ON_HF_URL if auth is None else SUBSCRIBE_TO_PRO_URL
+    text = SIGNUP_ON_HF_TXT if auth is None else SUBSCRIBE_TO_PRO_TXT
+    desc = unlogged_desc if auth is None else logged_desc
+    desc += f" {ending}."
+    style = ";".join([
+        "white-space: nowrap",
+        "text-underline-offset: 2px",
+        "color: var(--body-text-color)",
+    ])
+    if supports_html:
+        html += f'<a style="{style}" href="{link}">'
+    html += text
+    if supports_html:
+        html += '</a> '
+    html += desc
+    markdown = f'[{text}]({link}) {desc}'
+    return html, markdown
+
+
 def schedule(
     task_id: int,
     request: gr.Request | None = None,
@@ -57,7 +98,7 @@ def schedule(
     _first_attempt: bool = True,
 ) -> ScheduleResponse:
 
-    if not (gradio_version := version.parse(gr.__version__)).major == 4: # pragma: no cover
+    if not (gradio_version := version.parse(gr.__version__)).major >= 4: # pragma: no cover
         raise RuntimeError("ZeroGPU is only compatible with Gradio 4+")
 
     GRADIO_HTML_TOASTS = gradio_version.minor >= 39
@@ -75,29 +116,27 @@ def schedule(
     if isinstance(res, QuotaInfos): # pragma: no cover
         requested = duration.seconds if duration is not None else DEFAULT_SCHEDULE_DURATION
         if res.wait < timedelta(0):
-            message = (
+            raise gr.Error(
                 f"The requested GPU duration ({requested}s) "
                 f"is larger than the maximum allowed"
             )
         else:
+            gpu = "Pro GPU" if auth == 'pro' else ("free GPU" if auth == 'regular' else "GPU")
             message = (
-                f"You have exceeded your GPU quota "
-                f"({res.left}s left vs. {requested}s requested). "
+                f"You have exceeded your {gpu} quota "
+                f"({requested}s requested vs. {res.left}s left)."
             )
-            if not supports_auth() or auth == 'pro':
-                message += "Please"
-            elif auth is None or auth == 'regular':
-                link = SIGNUP_ON_HF_URL if auth is None else SUBSCRIBE_TO_PRO_URL
-                text = "Sign-up on Hugging Face" if auth is None else "Subscribe to Pro"
-                if GRADIO_HTML_TOASTS:
-                    message += f'<a href="{link}">'
-                message += text
-                if GRADIO_HTML_TOASTS:
-                    message += '</a>'
-                message += " to get more quotas or"
-            else:
-                assert_never(auth)
-        raise gr.Error(f"{message} retry in {res.wait}")
+            details_html, details_markdown = _toast_action(
+                auth=auth,
+                supports_html=GRADIO_HTML_TOASTS,
+                pro_message=f"Try again in {res.wait}",
+                unlogged_desc="to get more",
+                logged_desc="to get 5x more",
+                ending="usage quota",
+            )
+            message_html = f"{message} {details_html}"
+            message_text = f"{message} {details_markdown}"
+            raise HTMLError(html_string(message_html, message_text))
 
     if not isinstance(res, httpx.codes): # pragma: no cover
         gr.Info("Waiting for a GPU to become available")
@@ -120,7 +159,17 @@ def schedule(
                     raise RuntimeError("Connection closed by visitor while queueing")
                 continue
             if event.event == 'failed':
-                raise gr.Error(NO_GPU_MESSAGE_INQUEUE)
+                details_html, details_markdown = _toast_action(
+                    auth=auth,
+                    supports_html=GRADIO_HTML_TOASTS,
+                    pro_message="Retry later",
+                    unlogged_desc="to get a higher",
+                    logged_desc="to get the highest",
+                    ending="priority in ZeroGPU queues",
+                )
+                message_html = f"{NO_GPU_MESSAGE_INQUEUE}. {details_html}"
+                message_text = f"{NO_GPU_MESSAGE_INQUEUE} {details_markdown}"
+                raise HTMLError(html_string(message_html, message_text))
             if event.event == 'succeeded':
                 assert event.data is not None
                 if connection_event is not None and not connection_event.alive:
@@ -187,7 +236,4 @@ def _get_token(request: gr.Request | None) -> str | None:
     if not hasattr(headers, 'get'):
         headers = headers.__dict__ # pragma: no cover
 
-    if not (token := headers.get(TOKEN_HEADER.lower())):
-        raise gr.Error("Internal infra error")
-
-    return token
+    return headers.get(TOKEN_HEADER.lower())
