@@ -143,3 +143,8 @@ def one_launch(task: Callable[Param, None], *task_args: Param.args, **task_kwarg
         gr.Blocks.launch = _launch
         return gr.Blocks.launch(*args, **kwargs)
     gr.Blocks.launch = launch
+
+
+class HTMLError(gr.Error):
+    def __str__(self): # pragma: no cover
+        return self.message
