@@ -76,8 +76,11 @@ class Worker(Generic[Res]):
         self._sentinel = Thread(target=self._close_on_exit, daemon=True)
         self.arg_queue = Queue()
         self.res_queue = Queue()
+        debug(f"{self.arg_queue._writer.fileno()=}") # pyright: ignore [reportAttributeAccessIssue]
+        debug(f"{self.res_queue._writer.fileno()=}") # pyright: ignore [reportAttributeAccessIssue]
         if (server_port := get_server_port()) is not None:
             fds = [c.fd for c in psutil.Process().connections() if c.laddr.port == server_port]
+            debug(f"{fds=}")
         else:
             warnings.warn("Using a ZeroGPU function outside of Gradio caching or request might block the app")
             fds = []
@@ -200,9 +203,6 @@ def regular_function_wrapper(
         except PicklingError: # TODO: detailed serialization diagnostic
             release(fail=True)
             raise
-        # worker died before receiving arg data and Queue have been closed by sentinel
-        except (BrokenPipeError, TypeError): # Non-deterministic
-            pass
 
         while True:
             res = worker.res_queue.get()
@@ -231,20 +231,23 @@ def regular_function_wrapper(
     ):
         global forked
         forked = True
-        if (res := worker_init(
-            res_queue=res_queue,
-            allow_token=allow_token,
-            nvidia_uuid=nvidia_uuid,
-            fds=fds,
-        )) is not None: # pragma: no cover
-            res_queue.put(res)
-            return
         signal.signal(signal.SIGTERM, drop_params(arg_queue.close))
+        initialized = False
         while True:
             try:
                 (args, kwargs), gradio_context = arg_queue.get()
             except OSError:
                 break
+            if not initialized:
+                if (res := worker_init(
+                    res_queue=res_queue,
+                    allow_token=allow_token,
+                    nvidia_uuid=nvidia_uuid,
+                    fds=fds,
+                )) is not None:
+                    res_queue.put(res)
+                    return
+                initialized = True
             GradioPartialContext.apply(gradio_context)
             context = copy_context()
             with ThreadPoolExecutor() as executor:
@@ -311,9 +314,6 @@ def generator_function_wrapper(
         except PicklingError: # TODO: detailed serialization diagnostic
             release(fail=True)
             raise
-        # worker died before receiving arg data and Queue have been closed by sentinel
-        except (BrokenPipeError, TypeError): # Non-deterministic
-            pass
 
         yield_queue: ThreadQueue[YieldQueueResult[Res]] = ThreadQueue()
         def fill_yield_queue(worker: Worker[GeneratorResQueueResult[Res]]):
@@ -372,20 +372,23 @@ def generator_function_wrapper(
     ):
         global forked
         forked = True
-        if (res := worker_init(
-            res_queue=res_queue,
-            allow_token=allow_token,
-            nvidia_uuid=nvidia_uuid,
-            fds=fds,
-        )) is not None: # pragma: no cover
-            res_queue.put(res)
-            return
         signal.signal(signal.SIGTERM, drop_params(arg_queue.close))
+        initialized = False
         while True:
             try:
                 (args, kwargs), gradio_context = arg_queue.get()
             except OSError:
                 break
+            if not initialized:
+                if (res := worker_init(
+                    res_queue=res_queue,
+                    allow_token=allow_token,
+                    nvidia_uuid=nvidia_uuid,
+                    fds=fds,
+                )) is not None:
+                    res_queue.put(res)
+                    return
+                initialized = True
             def iterate():
                 gen = task(*args, **kwargs) # type: ignore
                 while True:
